% Clear
clear;
close all;
clc;

% Inputs
FMin = 1e9;
FMax = 2e9;
NSims = 100;
Sim_Array =(1:NSims);

%Parameters
V1Namelabel = 'AlN Thickness (\mum)';
V1Name = 'AlN Thickness (um)';
V1Beg = 180e-9;
V1End = 220e-9;
NV1 = 10;
V2Namelabel = 'Aratio';
V2Name = 'Aratio';
V2Beg = 0.4;
V2End = 0.6;
NV2 = 10;

% Load files
for a=1:NSims
fname = '/saw_unit_3D.flxhst';
sim = num2str(a);
dir = strcat(sim,fname);
Data = read_history(dir);

% Get samples, timestep and frequnecy vector
Ts = Data.TimeRecB.Time(3) - Data.TimeRecB.Time(2);
Nf = length(Data.TimeRecB.Time) - 1;
Fs = 1 / Ts;
Freq = linspace(0,Fs,Nf)';

% Create window
Win = 0.5 * (1 - cos(2 * pi * (1:Nf*2)' / (Nf*2 - 1)));
Win = Win(Nf + 1 : end);
% Win = ones(Nf,1);

% Extract data
V = Data.DataRecB(2).Data(1:end-1) .* Win;
I = diff(Data.DataRecB(3).Data) / Ts .* Win;

% FFT Data
VS = fft(V);
IS = fft(I);

% Calculate Z
Z = VS ./ IS;

% Apply FMax and Fmin to Z and Freq
IndLower = find(Freq>=FMin,1);
IndUpper = find(Freq>=FMax,1);
Freq = Freq(IndLower:IndUpper);
Freqint = linspace(FMin,FMax,Nf);
Z = Z(IndLower:IndUpper);

%Interpolate Z
Zint = interp1(Freq,Z,Freqint);

%Calculate Fp and Fs
[Zmin,IndMin] = min(Zint);
Fe(a) = Freqint(IndMin);
[Zmax,IndMax] = max(Zint);
Fm(a) = Freqint(IndMax);

%Calculate Conductance
for i=1:length(Zint)
Admt(i)=1/Zint(i);
Con(i) = real(Admt(i));
end
%Calculate Q
[ConPeak ConInd] = max(Con);
ConPeak2 = ConPeak / 2;
LowerCon = Con(1:ConInd);
UpperCon = Con(ConInd:length(Con));

[LowerAmp,LowerInd]=min(abs(LowerCon-ConPeak2));
[UpperAmp,UpperInd]=min(abs(UpperCon-ConPeak2));

F1 = Freqint(LowerInd);
F2 = Freqint(ConInd+UpperInd);
Q(a) = Freqint(ConInd) / (F2 - F1);

Impd(:,a)=abs(Zint(1:79609)');
end
%-----------------------------------------------

% Create vectors
V1Vect = linspace(V1Beg,V1End,NV1)';
V2Vect = linspace(V2Beg,V2End,NV2)';
[V1Grid,V2Grid] = meshgrid(V1Vect,V2Vect);
PlotSub = [1,1];
PlotInd = sub2ind([NV2,NV1],PlotSub(1),PlotSub(2));


% Create KPI Matrices
Fs = reshape(Fe,NV2,NV1);
Fp = reshape(Fm,NV2,NV1);
Q = reshape(Q,NV2,NV1);
JobNum = reshape(Sim_Array,NV2,NV1);

% Calculate Fp %
FpPC = 100 * (Fp / max(Fp(:)) - 1);

% Get max and min impedance for plotting
PZMin = 0.5 * min(min(Impd(1:end-1)));
PZMax = 2.0 * max(max(Impd(1:end-1)));

% Create figure and setup datatip
f = figure('Name','OnScale Post Processing','WindowStyle','normal','NumberTitle','off',...
           'units','normalized','outerposition',[0.05 0.075 0.9 0.9]);
cursorMode = datacursormode(f);
set(cursorMode, 'enable','on','UpdateFcn',@gettippos);

% Plot impedance
subplot(2,2,1);
hPlot = semilogy(Freqint(1:79609),Impd(:,PlotInd));
ZHand = gca;
xlim([FMin 1.79e9]);
title('Impedance');
xlabel('Frequency (MHz)');
ylabel('Impedance Magnitude (Ohms)');
ylim([PZMin PZMax]);

% Plot Q & add datatip
hax1 = subplot(2,2,2);
hSurf = surf(V1Vect,V2Vect,Q,'FaceColor','interp');
QHand = gca;
set(QHand,'Xdir','reverse','Ydir','reverse');
hDatatip = cursorMode.createDatatip(hSurf);
set(hDatatip,'Position',[V1Beg,V2Beg]);
title('Q');
xlabel(V1Namelabel);
ylabel(V2Namelabel);
zlabel('Q');
xlim([V1Beg V1End]);
ylim([V2Beg V2End]);

% Plot fs
hax2 = subplot(2,2,3);
surf(V1Vect,V2Vect,Fs,'FaceColor','interp');
FsHand = gca;
set(FsHand,'Xdir','reverse','Ydir','reverse');
title('Fs');
xlabel(V1Namelabel);
ylabel(V2Namelabel);
zlabel('Fs (MHz)');
xlim([V1Beg V1End]);
ylim([V2Beg V2End]);

% Plot Fp
hax3 = subplot(2,2,4);
surf(V1Vect,V2Vect,Fp,'FaceColor','interp');
FpHand = gca;
set(FpHand,'Xdir','reverse','Ydir','reverse');
title('Fp');
xlabel(V1Namelabel);
ylabel(V2Namelabel);
zlabel('Fp (MHz)');
xlim([V1Beg V1End]);
ylim([V2Beg V2End]);

% Link rotation on plots
hlink = linkprop([hax1,hax2,hax3],{'CameraPosition','CameraUpVector'}); 
% rotate3d on;

% Copy data to datatip to be accessible from update function
hSurf.UserData.Impd = Impd;
hSurf.UserData.V1Vect = V1Vect;
hSurf.UserData.V2Vect = V2Vect;
hSurf.UserData.V1Name = V1Name;
hSurf.UserData.V2Name = V2Name;
hSurf.UserData.hPlot = hPlot;
hSurf.UserData.PlotYLim = [PZMin PZMax];

% Create context menu
c = uicontextmenu;
hDatatip.UIContextMenu = c;

% Create child menu items for the uicontextmenu
m1 = uimenu(c,'Label','Show 2D Model Geometry','Callback',@setlinestyle);
m2 = uimenu(c,'Label','Show 3D Model Geometry','Callback',@setlinestyle);
m3 = uimenu(c,'Label','Export Results','Callback',@setlinestyle);

% Create context menu function
function setlinestyle(source,callbackdata)
     switch source.Label
        case 'Show 2D Model Geometry'
            WaitText = 'Loading 2D Results';
            waitfunc(WaitText);
        case 'Show 3D Model Geometry'
            WaitText = 'Loading 3D Results';
            waitfunc(WaitText);
            system('flexfea.exe saw3D.flxdato');
        case 'Export Results'
            WaitText = 'Exporting Results';
            waitfunc(WaitText);
    end
end

% Datatip update function
function TipText = gettippos(~,hTarg)

    % Get position
    TipPos = get(hTarg,'Position');
    
    % Get indices
    NV1 = length(hTarg.Target.UserData.V1Vect);
    NV2 = length(hTarg.Target.UserData.V2Vect);
    PlotSub(1) = find(hTarg.Target.UserData.V2Vect == TipPos(2),1,'first');
    PlotSub(2) = find(hTarg.Target.UserData.V1Vect == TipPos(1),1,'first');
    PlotInd = sub2ind([NV2,NV1],PlotSub(1),PlotSub(2));
      
    % Update plot
    set(hTarg.Target.UserData.hPlot,'YData',hTarg.Target.UserData.Impd(:,PlotInd)');
    % set(hTarg.Target.UserData.hPlot,'YLim',hTarg.Target.UserData.PlotYLim);
    
    % Create tip text
    TipText = {[hTarg.Target.UserData.V1Name ': ' num2str(TipPos(1))]; ...
                [hTarg.Target.UserData.V2Name ': ' num2str(TipPos(2))]; ...
                ['Q: ' num2str(hTarg.Target.ZData(PlotSub(1),PlotSub(2)),3)]};
    
end

% Create waitbar funciton
function waitfunc(WaitText)

    % Create waitbar
    w = waitbar(0,WaitText);
    
    % Loop through completion
    for Index = 1 : 100
        waitbar(Index/100,w);
        pause(0.02);
    end
    
    % Delete waitbar
    delete(w);

end
