function Hist = read_history(FName)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Title:          History Reader
%   Description:    Reads PZFlex time history into Matlab
%   Author:         Weidlinger
%   Date:           01/08/11
%   Version:        1.0
%
%   History:        1.0         Initial version
%                   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Open file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set precision of open / close bytes
OCForm = 'int32';
% Open the file
Fid = fopen(FName); % Replace FName with name of flex history file 
                    % e.g. 'FILENAME.flxhst'  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Headers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read header record 1 open byte
OpenBytes = fread(Fid,1,OCForm);
% Decide precision used in file
if (OpenBytes == 8)
    
    % Single precision
    IntForm = 'int32';
    FloatForm = 'float32';
    
elseif (OpenBytes == 12)
    
    % Double precision
    IntForm = 'int64';
    FloatForm = 'float64';
    
else
    
    % Unknown header format
    disp('Error: Unknown header format.');
    Hist = -1;
    return;
    
end;
% Read remainder of header record 1      
Header1.Label = fread(Fid,4,'*char')';
Header1.Type = fread(Fid,1,IntForm);
CloseBytes = fread(Fid,1,OCForm);
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.Header1 = Header1;
% Read header record 2
OpenBytes = fread(Fid,1,OCForm);             
Header2.FType = fread(Fid,20,'*char')';
Header2.FForm = fread(Fid,20,'*char')';
Header2.Date = fread(Fid,80,'*char')';
Header2.Code = fread(Fid,20,'*char')';
Header2.User = fread(Fid,200,'*char')';
Header2.Extra1 = fread(Fid,20,'*char')';
Header2.Extra2 = fread(Fid,20,'*char')';
CloseBytes = fread(Fid,1,OCForm); 
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.Header2 = Header2;
% Read header record 3
OpenBytes = fread(Fid,1,OCForm);             
Header3.Title = fread(Fid,200,'*char')';
Header3.Tag = fread(Fid,80,'*char')';
CloseBytes = fread(Fid,1,OCForm); 
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.Header3 = Header3;
% Read header record 4
OpenBytes = fread(Fid,1,OCForm);             
Header4.Version = fread(Fid,1,IntForm);
Header4.NPartitions = fread(Fid,1,IntForm);
Header4.IPartition = fread(Fid,1,IntForm); 
Header4.IntBytes = fread(Fid,1,IntForm);
Header4.RealBytes = fread(Fid,1,IntForm);
Header4.Char1Bytes = fread(Fid,1,IntForm);
Header4.Char2Bytes = fread(Fid,1,IntForm);
Header4.Char3Bytes = fread(Fid,1,IntForm); 
Header4.Extra1 = fread(Fid,1,IntForm); 
Header4.Extra2 = fread(Fid,1,IntForm); 
CloseBytes = fread(Fid,1,OCForm); 
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.Header4 = Header4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Base record
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read base record
OpenBytes = fread(Fid,1,OCForm);             
BaseRec.In12 = fread(Fid,1,IntForm);
BaseRec.NRecords = fread(Fid,1,IntForm);
BaseRec.NTimes = fread(Fid,1,IntForm);
BaseRec.NGrid = fread(Fid,1,IntForm);
BaseRec.NDescription = fread(Fid,1,IntForm);
BaseRec.NTag = fread(Fid,1,IntForm);
BaseRec.NTShift = fread(Fid,1,IntForm); 
BaseRec.NLabel = fread(Fid,1,IntForm); 
BaseRec.NIndex = fread(Fid,1,IntForm); 
BaseRec.NCoord = fread(Fid,1,IntForm); 
BaseRec.NChar = fread(Fid,1,IntForm); 
BaseRec.NExtra = fread(Fid,1,IntForm); 
CloseBytes = fread(Fid,1,OCForm); 
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.BaseRec = BaseRec;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time Records
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read time record A
OpenBytes = fread(Fid,1,OCForm);             
TimeRecA.Label = fread(Fid,20,'*char')';
TimeRecA.Dummy1 = fread(Fid,20,'*char')';
TimeRecA.Dummy2 = fread(Fid,20,'*char')';
TimeRecA.Description = fread(Fid,80,'*char')';
CloseBytes = fread(Fid,1,OCForm); 
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.TimeRecA = TimeRecA;
% Read time record B
OpenBytes = fread(Fid,1,OCForm);             
TimeRecB.Zero = fread(Fid,2,FloatForm);
TimeRecB.TStep = fread(Fid,1,FloatForm);
TimeRecB.IntZero = fread(Fid,3,IntForm);
TimeRecB.TBegin = fread(Fid,1,FloatForm);
TimeRecB.TEnd = fread(Fid,1,FloatForm);
TimeRecB.TShift = fread(Fid,1,FloatForm);
TimeRecB.Time = fread(Fid,BaseRec.NTimes,FloatForm);
CloseBytes = fread(Fid,1,OCForm); 
% Check header and write to main structure
if (OpenBytes ~= CloseBytes)    
    Hist = -1;
    return;   
end;
% Save to structure
Hist.TimeRecB = TimeRecB;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data records
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Loop through data records
for Index = 1 : ( BaseRec.NRecords - 1 )
    % Read data record A
    OpenBytes = fread(Fid,1,OCForm);
    DataRecA(Index).Label = fread(Fid,20,'*char')';
    DataRecA(Index).CurveTag = fread(Fid,20,'*char')';
    DataRecA(Index).GridTag = fread(Fid,20,'*char')';
    DataRecA(Index).Description = fread(Fid,80,'*char')';
    CloseBytes = fread(Fid,1,OCForm);
    
    % Check header
    if (OpenBytes ~= CloseBytes)
        Hist = -1;
        return;
    end;
    
    % Read data record B
    OpenBytes = fread(Fid,1,OCForm);
    DataRecB(Index).XYZ = fread(Fid,3,FloatForm);
    DataRecB(Index).IJK = fread(Fid,3,IntForm);
    DataRecB(Index).DataMin = fread(Fid,1,FloatForm);
    DataRecB(Index).DataMax = fread(Fid,1,FloatForm);
    DataRecB(Index).TShift = fread(Fid,1,FloatForm);
    DataRecB(Index).Data = fread(Fid,BaseRec.NTimes,FloatForm);
    CloseBytes = fread(Fid,1,OCForm);
    
    % Check header
    if (OpenBytes ~= CloseBytes)
        Hist = -1;
        return;
    end;
end;
% Save to structure
Hist.DataRecA = DataRecA';
Hist.DataRecB = DataRecB';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Close
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Close the file
fclose(Fid);